// kaldifst/python/csrc/context-fst.h
//
// Copyright (c)  2023  Xiaomi Corporation (authors: Fangjun Kuang)

#ifndef KALDIFST_PYTHON_CSRC_CONTEXT_FST_H_
#define KALDIFST_PYTHON_CSRC_CONTEXT_FST_H_

#include "kaldifst/python/csrc/kaldifst.h"

namespace kaldifst {

void PybindContextFst(py::module *m);

}  // namespace kaldifst

#endif  //  KALDIFST_PYTHON_CSRC_CONTEXT_FST_H_
